#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 20 18:34:11 2018

@author: patrickmcfarlane

team.py contains the Team class that
enables API calls for team-related
endpoints
"""

from .utils import api_call, parse_api_call

class Team:
    """ The Team class contains all resources needed
    to use the team-related API calls. `stats.nba.com <https://stats.nba.com>`_
    has the following team-related API endpoints:

        - **teamdashboardbyclutch**: Traditional and rank statistics broken \
        down by different definitions of clutch for a team.
        - **teamdashboardbygamesplits**: Traditional and rank statistics \
        broken down by different splits (half, quarter, and score \
        differential).
        - **teamdashboardbygeneralsplits**: Traditional and rank statistics \
        broken down by different splits (win/loss, location, month, \
        pre/post All-Star, days rest). 
        - **teamdashboardbylastngames**: Traditional and rank statistics \
        broken down by the number of n recent games and game number \
        bins.
        - **teamdashboardbyopponent**: Traditional and rank statistics \
        broken down by opponent splits (conference, division, and \
        individual team).
        - **teamdashboardbyshootingsplits**: Traditional and rank statistics \
        broken down by shooting splits (shot distance, shot area, \
        assisted/unassisted, shot type, and indivdual assistant).
        - **teamdashboardbyteamperformance**: Traditional and rank statistics \
        broken down by team performance splits (win/loss, score differential, \
        points for, and points against).
        - **teamdashboardbyyearoveryear**: Traditional and rank statistics \
        broken down by year.
        - **teamdashlineups**: Traditional and plus/minus statistics \
        for sets of lineups between sizes 2 to 5 players, inclusive.
        - **teamdashptpass**: Shooting statistics for passes to and from \
        a player broken down by teammates.
        - **teamdashptreb**: Rebound statistics broken down by shot type, \
        contesting players, and shot/rebound distance.
        - **teamdashptshots**: Shooting statistics broken down by shot \
        type, shot clock time, number of tribbles, defender proximity, and \
        length of touch.
        - **teamgamelog**: Game log statistics for a given year.
        - **teaminfocommon**: Team information for a given year.
        - **teamplayerdashboard**: Player traditional and rank statistics \
        for a given team.
        - **teamplayeronoffdetails**: Team traditional and rank statistics \
        broken down by on/off splits per player.
        - **teamplayeronoffsummary**: Team summary statistics broken down \
        by on/off splits per player.
        - **teamvsplayer**: Team statistics versus a given opponent player \
        broken down by several shooting related splits (shot distance and \
        area)
        - **teamyearbyyearstats**: Team statistics and performance broken \
        down by year.

    The Team class has the following required parameters:

        @param **headers** (*dict*): Dictionary of request header information
            required by the API. Specifically, the API requires you to declare
            the 'User-Agent' key of the request header dictionary. More information
            can be found `here <https://stackoverflow.com/questions/46781563/
            how-to-obtain-a-json-response-from-the-stats-nba-com-api>`_ and
            an example request header dictionary can be found in the __init__.py
            file in the tests folder of this module.

        @param **league_id** (*str*): LeagueID in the API. String of a \
            two-digit number corresponding to the league. '00' is the NBA, \
            '10' is the WNBA, '01' is the ABA, and '20' is the G-League.

        @param **per_mode** (*str*): PerMode in the API. String indicating \
            the type of rate stats to be returned. Valid values include:

                - 'Totals', 'PerGame', 'MinutesPer', 'Per48', 'Per40', \
                'Per36', 'PerMinute', 'PerPossession', 'PerPlay', \
                'Per100Possessions', 'Per100Plays'

        @param **plus_minus** (*str*): PlusMinus in the API. String \
            representing a Boolean value that indicates whether the \
            values being returned should be in plus-minus form. \
            Valid values include:

                - 'Y', 'N'

        @param **rank** (*str*): Rank in the API. String representing \
            a Boolean value that indicates whether the values being \
            returned should be in rank form. Valid values include:

                - 'Y', 'N'

        @param **pace_adjust** (*str*): PaceAdjust in the API. String \
            representing a Boolean value that indicates whether the \
            values being returned should be pace-adjusted. \
            Valid values include:

                - 'Y', 'N'

        @param **measure_type** (*str*): MeasureType in the API. String \
            indicating the set of statistics to be returned. Valid values \
            include:

                - 'Base', 'Advanced', 'Misc', 'Four Factors', 'Scoring', \
                'Opponent', 'Usage', 'Defense'

        @param **period** (*str*): Period in the API. String of an integer \
            value that corresponds to a desired quarter for data to be \
            returned. A value of '0' returns data across all quarters.

        @param **vs_conference** (*str*): VsConference in the API. String \
            indicating the conference of the opposing team for data to be \
            returned. An empty string returns data across all conferences. \
            Valid values include:

                - 'East', 'West', ''

        @param **last_n_games** (*str*): LastNGames in the API. String of \
            an integer indicating the desired number of most recent games \
            for data to be returned. A value of '0' returns data across \
            all previous games, subject to other constraints in the API call.

        @param **team_id** (*str*): TeamID in the API. String of a 10-digit \
            integer that uniquely identifies a team for which data \
            is to be returned.

        @param **location** (*str*): Location in the API. String indicating \
            the game location for the data to be returned. An empty string \
            returns data across both home and road games. Valid values \
            include:

                - 'Home', 'Road', ''

        @param **outcome** (*str*): Outcome in the API. String indicating \
            the game outcome for the data to be returned. An empty string \
            returns data across both wins and losses. Valid values include:

                - 'W', 'L', ''

        @param **date_from** (*str*): DateFrom in the API. String of a date \
            in a MM/DD/YYYY format indicating the start date for which \
            data is to be returned.

        @param **date_to** (*str*): DateTo in the API. String of a date \
            in a MM/DD/YYYY format indicating the end date for which \
            data is to be returned.

        @param **opp_team_id** (*str*): OpponentTeamID in the API. String \
            of a 10-digit integer that uniquely identifies an opposing \
            team for which data is to be returned.

        @param **season** (*str*): Season in the API. String of a two-year \
            season in a YYYY-ZZ format, where the ZZ are the last two \
            digits of the following year. For example, '2017-18' is a valid \
            value of **season** and represents the 2017-18 NBA season.

        @param **vs_division** (*str*): VsDivision in the API. String \
            indicating the division of the opposing team for data to be \
            returned. An empty string returns data across all divisions. \
            Valid values include:

                - 'Atlantic', 'Central', 'Northwest', 'Pacific', \
                'Southeast', 'Southwest', 'East', 'West',  ''

            The 'East' and 'West' values correspond to conferences.

        @param **game_segment** (*str*): GameSegment in the API. String \
            indicating the section of a game for data to be returned. \
            An empty string returns data across all game segments. Valid \
            values include:

                - 'First Half', 'Overtime', 'Second Half', ''

        @param **month** (*str*): Month in the API. String of an integer \
            corresponding to a month for data to be returned. A value \
            of '0' returns data across all months.

        @param **season_type** (*str*): SeasonType in the API. String \
            indicating the type of season for data to be returned. \
            Valid values include:

                - 'Regular Season', 'Pre Season', 'Playoffs', 'All Star'

        @param **season_segment** (*str*): SeasonSegment in the API): String \
            indicating the section of the season for data to be returned. \
            An empty string returns data across all season segments. \
            Valid values include:

                - 'Pre All-Star', 'Post All-Star', ''

        @param **vs_player_id** (*str*): VsPlayerID in the API. String of \
            an integer corresponding to a player ID for a given player.

        @param **game_id** (*str*): GameID in the API. 10-digit string \
            that represents a unique game. The format is two leading zeroes, \
            followed by a season indicator number ('1' for preseason, \
            '2' for regular season, '4' for the post-season), \
            then the trailing digits of the season in which the game \
            took place (e.g. '17' for the 2017-18 season). The following \
            5 digits increment from '00001' in order as the season progresses. \
            For example, '0021600001' is the **game_id** of the first game \
            of the 2016-17 NBA regular season.

        @param **group_quantity** (*str*): GroupQuantity in the API. String \
            of an integer indicating the number of players to include a \
            lineup for the **leaguedashlineups** endpoint. The minimum value \
            is '2' and the maximum value is '5'.

    Attributes:

        **api_resp** (*dict*): JSON object of the API response. The API \
            response has three keys. The 'resource' key describes the \
            type of response returned (the endpoint in this instance). \
            The 'parameters' key describes the parameters provided in \
            the API call. The 'resultSets' key contains the data returned \
            in the API call.

        **data** (*dict*): A dictionary of response names. Each response \
            name is a key to a list of dictionaries containing the \
            corresponding data.
    """

    def __init__(self, headers, endpoint='teamdashboardbyclutch',
                 player_id='2544',
                 league_id='00',
                 per_mode='PerGame', plus_minus='N',
                 rank='Y', pace_adjust='N',
                 measure_type='Base', period='0',
                 vs_conference='', last_n_games='0',
                 team_id='1610612747', location='', outcome='',
                 date_from='', date_to='', opp_team_id='0',
                 season='2017-18', vs_division='',
                 game_segment='', month='0',
                 season_type='Regular Season', season_segment='',
                 vs_player_id='201939',
                 game_id='0011800079', group_quantity='5'):

        # Controlling the parameters depending on the endpoint
        if endpoint not in ['teamgamelog', 'teaminfocommon',
                            'teamyearbyyearstats']:
            params = {'LeagueID': league_id,
                      'PerMode': per_mode,
                      'PlusMinus': plus_minus,
                      'Rank': rank,
                      'PaceAdjust': pace_adjust,
                      'MeasureType': measure_type,
                      'Period': period,
                      'VsConference': vs_conference,
                      'Location': location,
                      'Outcome': outcome,
                      'DateFrom': date_from,
                      'DateTo': date_to,
                      'TeamID': team_id,
                      'OpponentTeamID': opp_team_id,
                      'Season': season,
                      'VsDivision': vs_division,
                      'GameSegment': game_segment,
                      'Month': month,
                      'SeasonType': season_type,
                      'SeasonSegment': season_segment,
                      'LastNGames': last_n_games}
        elif endpoint in ['teamgamelog', 'teaminfocommon']:
            params = {'LeagueID': league_id,
                      'TeamID': team_id,
                      'Season': season,
                      'SeasonType': season_type}
        elif endpoint in ['teamyearbyyearstats']:
            params = {'LeagueID': league_id,
                      'TeamID': team_id,
                      'PerMode': per_mode,
                      'SeasonType': season_type}

        if endpoint in ['teamdashlineups']:
            params['GroupQuantity'] = group_quantity
            params['GameID'] = game_id
        elif endpoint in ['teamdashptpass',
                          'teamdashptreb',
                          'teamdashptshots']:
            del params['PlusMinus'], params['PaceAdjust']
            del params['Rank'], params['MeasureType']
            if endpoint == 'teamdashptpass':
                del params['Period'], params['GameSegment']
        elif endpoint in ['teamvsplayer']:
            params['VsPlayerID'] = vs_player_id

        self.api_resp = api_call(endpoint=endpoint,
                                 params=params,
                                 headers=headers)

        # Storing the API response in a dictionary called data
        # The results can come in different formats, namely a
        # dictionary keyed by either resultSets or resultSet
        self.data = parse_api_call(self.api_resp)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Oct 11 20:44:34 2018

@author: patrickmcfarlane

league_dash.py contains the LeagueDashboard class that
enables API calls for general league performance statitics
related endpoints
"""

from .utils import api_call, parse_api_call

class LeagueDash:
    """ The LeagueDash class contains all resources needed
    to use the league performance stats related API calls.
    `stats.nba.com <https://stats.nba.com>`_ has the following
    league performance stats related API endpoints:

        *- **leaguedashlineups**: Traditional and plus/minus statistics \
        for sets of lineups between sizes 2 to 5 players, inclusive.
        - **leaguedashplayerbiostats**: Player metadata and performance \
        statistics for a given season.
        *- **leaguedashplayerclutch**: Traditional, plus/minus, and rank \
        statistics for players in a defined clutch period.
        *- **leaguedashplayershotlocations**: Player shooting-related \
        statistics by shot distance/type.
        - **leaguedashplayerptshot**: Shooting-related statistics for \
        a given season by player.
        *- **leaguedashplayerstats**: Traditional, plus/minus, and rank \
        statistics for players.
        - **leaguedashptdefend**: Defensive statistics for a given season \
        by player.
        - **leaguedashptteamdefend**: Defensive statistics for a given \
        season by team.
        - **leaguedashteamptshot**: Shooting-related statistics for a \
        given season by team.

    The LeagueDash class has the following required parameters:

        @param **headers** (*dict*): Dictionary of request header information
            required by the API. Specifically, the API requires you to declare
            the 'User-Agent' key of the request header dictionary. More information
            can be found `here <https://stackoverflow.com/questions/46781563/
            how-to-obtain-a-json-response-from-the-stats-nba-com-api>`_ and
            an example request header dictionary can be found in the __init__.py
            file in the tests folder of this module.

        @param **league_id** (*str*): LeagueID in the API). String of a \
            two-digit number corresponding to the league. '00' is the NBA, \
            '10' is the WNBA, '01' is the ABA, and '20' is the G-League.

        @param **group_quantity** (*str*): GroupQuantity in the API. String \
            of an integer indicating the number of players to include a \
            lineup for the **leaguedashlineups** endpoint. The minimum value \
            is '2' and the maximum value is '5'.

        @param **per_mode** (*str*): PerMode in the API. String indicating \
            the type of rate stats to be returned. Valid values include:

                - 'Totals', 'PerGame', 'MinutesPer', 'Per48', 'Per40', \
                'Per36', 'PerMinute', 'PerPossession', 'PerPlay', \
                'Per100Possessions', 'Per100Plays'

        @param **plus_minus** (*str*): PlusMinus in the API. String \
            representing a Boolean value that indicates whether the values \
            being returned should be in plus-minus form. Valid values \
            include:

                - 'Y', 'N'

        @param **rank** (*str*): Rank in the API. String representing \
            a Boolean value that indicates whether the values being \
            returned should be in rank form. Valid values include:

                - 'Y', 'N'

        @param **pace_adjust** (*str*): PaceAdjust in the API. String \
            representing a Boolean value that indicates whether the \
            values being returned should be pace-adjusted. Valid \
            values include:

                - 'Y', 'N'

        @param **measure_type** (*str*): MeasureType in the API. String \
            indicating the set of statistics to be returned. Valid \
            values include:

                - 'Base', 'Advanced', 'Misc', 'Four Factors', 'Scoring', \
                'Opponent', 'Usage', 'Defense'

        @param **period** (*str*): Period in the API. String of an \
            integer value that corresponds to a desired quarter for data \
            to be returned. A value of '0' returns data across all quarters.

        @param **vs_conference** (*str*): VsConference in the API. String \
            indicating the conference of the opposing team for data to be \
            returned. An empty string returns data across all conferences. \
            Valid values include:

                - 'East', 'West', ''

        @param **last_n_games** (*str*): LastNGames in the API. String of \
            an integer indicating the desired number of most recent games \
            for data to be returned. A value of '0' returns data across \
            all previous games, subject to other constraints in the API call.

        @param **team_id** (*str*): TeamID in the API. String of a 10-digit \
            integer that uniquely identifies a team for which data is to \
            be returned.

        @param **location** (*str*): Location in the API. String indicating \
            the game location for the data to be returned. An empty string \
            returns data across both home and road games. Valid values \
            include:

                - 'Home', 'Road', ''

        @param **outcome** (*str*): Outcome in the API. String indicating \
            the game outcome for the data to be returned. An empty string \
            returns data across both wins and losses. Valid values include:

                - 'W', 'L', ''

        @param **date_from** (*str*): DateFrom in the API. String of a date \
            in a MM/DD/YYYY format indicating the start date for which \
            data is to be returned.

        @param **date_to** (*str*): DateTo in the API. String of a date \
            in a MM/DD/YYYY format indicating the end date for which \
            data is to be returned.

        @param **opp_team_id** (*str*): OpponentTeamID in the API. String \
            of a 10-digit integer that uniquely identifies an opposing \
            team for which data is to be returned.

        @param **season** (*str*): Season in the API. String of a two-year \
            season in a YYYY-ZZ format, where the ZZ are the last two \
            digits of the following year. For example, '2017-18' is a valid \
            value of **season** and represents the 2017-18 NBA season.

        @param **vs_division** (*str*): VsDivision in the API. String \
            indicating the division of the opposing team for data to be \
            returned. An empty string returns data across all divisions. \
            Valid values include:

                - 'Atlantic', 'Central', 'Northwest', 'Pacific', \
                'Southeast', 'Southwest', 'East', 'West',  ''

            The 'East' and 'West' values correspond to conferences.

        @param **game_segment** (*str*): GameSegment in the API. String \
            indicating the section of a game for data to be returned. \
            An empty string returns data across all game segments. \
            Valid values include:

                - 'First Half', 'Overtime', 'Second Half', ''

        @param **month** (*str*): Month in the API. String of an integer \
            corresponding to a month for data to be returned. \
            A value of '0' returns data across all months.

        @param **season_type** (*str*): SeasonType in the API. String \
            indicating the type of season for data to be returned. \
            Valid values include:

                - 'Regular Season', 'Pre Season', 'Playoffs', 'All Star'

        @param **season_segment** (*str*): SeasonSegment in the API. String \
            indicating the section of the season for data to be returned. \
            An empty string returns data across all season segments. \
            Valid values include:

                - 'Pre All-Star', 'Post All-Star', ''

        @param **clutch_time** (*str*): ClutchTime in the API. String that \
            defines the type of clutch time for the data to be returned. \
            Valid values include:

                - 'Last 5 Minutes', 'Last 4 Minutes', 'Last 3 Minutes', \
                'Last 2 Minutes', 'Last 1 Minute', 'Last 30 Seconds', \
                'Last 10 Seconds'

        @param **ahead_behind** (*str*): AheadBehind in the API. String \
            indicating the type of score differential for the data to \
            be returned. Valid values include:

                - 'Ahead or Behind', 'Behind or Tied', 'Ahead or Tied'

        @param **point_diff** (*str*): PointDiff in the API. String of zero \
            or a positive integer indicating the maximum point \
            differential for data to be returned. 

        @param **game_scope** (*str*): GameScope in the API. String \
            indicating the recency of the data to be returned. An \
            empty string returns data across all past games, subject \
            to other constraints in the API call. Valid values include:

                - 'Yesterday', 'Last 10', ''

        @param **player_experience** (*str*): PlayerExperience in the API. \
            String indicating the level of player experience for data to be \
            returned. An empty string returns data across all levels \
            of player experience. Valid values include:

                - 'Rookie', 'Sophomore', 'Veteran', ''

        @param **player_position** (*str*): PlayerPosition in the API. \
            String indicating the player position for data to be returned. \
            An empty string returns data across all player positions. Valid \
            values include:

                - 'F', 'C', 'G', 'C-F', 'F-C', 'F-G', 'G-F', ''

        @param **starters_bench** (*str*): StarterBench in the API. String \
            indicating whether data should be returned for either or both \
            starters or bench players. An empty string returns data across \
            both starters and bench players. Valid values include:

                - 'Starters', 'Bench', ''

        @param **distance_range** (*str*): DistanceRange in the API. String \
            indicating the size/type of the distance range bins for data \
            to be returned. Valid values include:

                - '5ft Range', '8ft Range', 'By Zone'

        @param **defense_category** (*str*): DefenseCategory in the API. \
            String indicating the shot type of defensive data to be returned. \
            Valid values include:

                - 'Overall', '3 Pointers', '2 Pointers', 'Less Than 6Ft', \
                'Less Than 10Ft', 'Greater Than 15Ft'

    Attributes:

        **api_resp** (*dict*): JSON object of the API response. The API \
            response has three keys. The 'resource' key describes the \
            type of response returned (the endpoint in this instance). \
            The 'parameters' key describes the parameters provided in the \
            API call. The 'resultSets' key contains the data returned in \
            the API call.

        **data** (*dict*): A dictionary of response names. Each response \
            name is a key to a list of dictionaries containing the \
            corresponding data.
    """

    def __init__(self, headers, endpoint='leaguedashlineups',
                 league_id='00', group_quantity='5',
                 per_mode='PerGame', plus_minus='N',
                 rank='Y', pace_adjust='N',
                 measure_type='Base', period='0',
                 vs_conference='', last_n_games='0',
                 team_id='0', location='', outcome='',
                 date_from='', date_to='', opp_team_id='0',
                 season='2017-18', vs_division='',
                 game_segment='', month='0',
                 season_type='Regular Season', season_segment='',
                 clutch_time='Last 5 Minutes',
                 ahead_behind='Ahead or Behind',
                 point_diff='0', game_scope='',
                 player_experience='',
                 player_position='', starters_bench='',
                 distance_range='By Zone',
                 defense_category='Overall'):

        # Controlling the parameters depending on the endpoint
        if endpoint not in ['leaguedashplayerbiostats',
                            'leaguedashplayerptshot',
                            'leaguedashteamptshot',
                            'leaguedashptdefend',
                            'leaguedashptteamdefend']:
            params = {'LeagueID': league_id,
                      'PerMode': per_mode,
                      'PlusMinus': plus_minus,
                      'Rank': rank,
                      'PaceAdjust': pace_adjust,
                      'MeasureType': measure_type,
                      'Period': period,
                      'VsConference': vs_conference,
                      'Location': location,
                      'Outcome': outcome,
                      'DateFrom': date_from,
                      'DateTo': date_to,
                      'TeamID': team_id,
                      'OpponentTeamID': opp_team_id,
                      'Season': season,
                      'VsDivision': vs_division,
                      'GameSegment': game_segment,
                      'Month': month,
                      'SeasonType': season_type,
                      'SeasonSegment': season_segment,
                      'LastNGames': last_n_games,
                      'GameScope': game_scope,
                      'PlayerExperience': player_experience,
                      'PlayerPosition': player_position,
                      'StarterBench': starters_bench}
        else:
            params = {'LeagueID': league_id,
                      'PerMode': per_mode,
                      'Season': season,
                      'SeasonType': season_type}

        if endpoint in ['leaguedashlineups']:
            params['GroupQuantity'] = group_quantity
        elif endpoint in ['leaguedashplayerclutch',
                          'leaguedashteamclutch']:
            params['ClutchTime'] = clutch_time
            params['AheadBehind'] = ahead_behind
            params['PointDiff'] = point_diff
        elif endpoint in ['leaguedashplayershotlocations',
                          'leaguedashteamshotlocations']:
            params['DistanceRange'] = distance_range
        elif endpoint in ['leaguedashptdefend',
                          'leaguedashptteamdefend']:
            params['DefenseCategory'] = defense_category


        self.api_resp = api_call(endpoint=endpoint,
                                 params=params,
                                 headers=headers)

        # Storing the API response in a dictionary called data
        # The results can come in different formats, namely a
        # dictionary keyed by either resultSets or resultSet
        self.data = parse_api_call(self.api_resp)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 10 19:59:39 2018

@author: patrickmcfarlane

league.py contains the League class that
enables API calls for general league related endpoints
"""

from .utils import api_call, parse_api_call

class League:
    """ The League class contains all resources needed to use the league-
    related API calls. `stats.nba.com <https://stats.nba.com>`_
    has the following league-related API endpoints:

        - **commonallplayers**: Current roster information if the \
        **current_season** flag is '1', historical player information \
        if the **current_season** flag is '0'.
        - **commonteamyears**: Start and end dates for teams in league \
        history.
        - **commonplayoffseries**: Playoff series matchup breakdown by game.
        - **franchisehistory**: Current and defunct franchise histories, \
        including performance and franchise metadata.
        - **playoffpicture**: Current state of the playoff picture by \
        conference.

    The League class has the following required parameters:

        @param **headers** (*dict*): Dictionary of request header information
            required by the API. Specifically, the API requires you to declare
            the 'User-Agent' key of the request header dictionary. More information
            can be found `here <https://stackoverflow.com/questions/46781563/
            how-to-obtain-a-json-response-from-the-stats-nba-com-api>`_ and
            an example request header dictionary can be found in the __init__.py
            file in the tests folder of this module.

        @param **league_id** (*str*): LeagueID in the API. String of a \
            two-digit number corresponding to the league. '00' is the NBA, \
            '10' is the WNBA, '01' is the ABA, and '20' is the G-League.

        @param **season** (*str*): Season in the API. String of a two-year
            season in a YYYY-ZZ format, where the ZZ are the last two \
            digits of the following year. For example, '2017-18' is a valid \
            value of **season** and represents the 2017-18 NBA season. \
            **season** is required by the 'commonallplayers' and \
            'commonplayoffseries' endpoints.

        @param **season_id** (*str*): SeasonID in the API. String of a year \
            season in a XYYYY format. X indicates the season type \
            ('1' for preseason, '2' for regular season, '4' for the playoffs). \
            For example, '22017' is a valid \
            value of **season_id** and represents the 2017-18 NBA regular
            season. **season_id** is required by the 'playoffpicture' \
            endpoint.

        @param **current_season** (*str*): IsOnlyCurrentSeason in the API. \
            Boolean value ('1' or '0') indicating whether only the current \
            season should be returned ('1'). A value of '0' returns all \
            players in league history. **current_season** is required by the \
            'commonallplayers' endpoint.

    Attributes:

        **api_resp** (*dict*): JSON object of the API response. The API \
            response has three keys. The 'resource' key describes the type of \
            response returned (the endpoint in this instance). The 'parameters' \
            key describes the parameters provided in the API call. The \
            'resultSets' key contains the data returned in the API call.

        **data** (*dict*): A dictionary of response names. Each response \
            name is a key to a list of dictionaries containing the \
            corresponding data.
    """

    def __init__(self, headers, endpoint='commonallplayers',
                 league_id='00', season='2017-18',
                 season_id='22017', current_season='1'):

        # Controlling the parameters depending on the endpoint
        if endpoint in ['commonteamyears', 'franchisehistory']:
            params = {'LeagueID': league_id}
        elif endpoint in ['commonplayoffseries']:
            params = {'LeagueID': league_id,
                      'Season': season}
        elif endpoint in ['playoffpicture']:
            params = {'LeagueID': league_id,
                      'SeasonID': season_id}
        else:
            params = {'LeagueID': league_id,
                      'Season': season,
                      'IsOnlyCurrentSeason': current_season}

        self.api_resp = api_call(endpoint=endpoint,
                                 params=params,
                                 headers=headers)

        # Storing the API response in a dictionary called data
        # The results can come in different formats, namely a
        # dictionary keyed by either resultSets or resultSet
        self.data = parse_api_call(self.api_resp)





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Oct 21 16:06:31 2018

@author: patrickmcfarlane

utils.py

Utilities for the py_ball package
"""

from requests import get

BASE_URL = 'http://stats.nba.com/stats/{endpoint}/'
BASE_WNBA_URL = 'http://data.wnba.com/data/5s/v2015/json/mobile_teams/' + \
    'wnba/{season}/scores/pbp/{game_id}_{quarter}_pbp.json'

def api_call(endpoint, params, headers):
    """ This function completes the API call at the given
    end point with the provided parameters.

    Args:
        - @param **endpoint** (*str*): string corresponding to a \
            `stats.nba.com <https://stats.nba.com>`_ API endpoint

    Returns:
        - JSON object of the API response
    """

    api_response = get(BASE_URL.format(endpoint=endpoint), params=params,
                       headers=headers)

    api_response.raise_for_status()
    json_resp = api_response.json()

    api_response.close()
    return json_resp


def wnba_shot_call(params, headers):
    """ This function completes the API call for WNBA
    shot data with the provided parameters.

    Args:
        - @param **params** (*str*): Dictionary containing required
            parameters for the WNBA shot data URL

    Returns:
        - **pbp_list** (*list*): list of play-by-play data
    """

    pbp_list = []
    for i in range(1, 10):
        try:
            api_response = get(BASE_WNBA_URL.format(season=params['season'],
                                                    game_id=params['game_id'],
                                                    quarter=i,
                                                    headers=headers))
        
            api_response.raise_for_status()
            json_resp = api_response.json()
            pbp_list += json_resp['g']['pla']
        except:
            break

    api_response.close()
    return pbp_list


def parse_api_call(api_resp):
    """ This function parses the API call returned from **api_call**
    and stores the response in a dictionary.

    Args:
        - @param **api_resp** (*dict*): JSON object of an API response. \
        This dictionary is keyed by 'resource', 'parameters', and \
        'resultSets'/'resultSet'. 'resource' contains the endpoint of the \
        API call. 'parameters' contains the parameters passed to the API call. \
        'resultSets'/'resultSet' contains the data returned from the \
        API call.

    Returns:
        - Dictionary keyed by all table names returned from the API call \
        containing all data in 'resultSets'/'resultSet'.
    """

    data = {}
    if 'resultSets' in api_resp:
        dictionary_key = 'resultSets'
    elif 'resultSet' in api_resp:
        dictionary_key = 'resultSet'

    if isinstance(api_resp[dictionary_key], list):
        for result_set in api_resp[dictionary_key]:
            headers = result_set['headers']
            if len(headers) > 0:
                if isinstance(headers[0], dict):
                    add_on = headers[0]['columnNames']
                    keep_header = headers[1]['columnNames']
                    col_ind = 0
                    col_count = 0
                    add_count = 0
                    for col_name in keep_header:
                        col_count += 1
                        if col_count <= 5:
                            continue
                        else:
                            keep_header[col_count] += '_' + add_on[col_ind]
                            add_count += 1
                            if add_count == 2:
                                add_count = 0
                                col_ind = 1
                    headers = keep_header
            values = result_set['rowSet']
            name = result_set['name']
            data[name] = [dict(zip(headers, value)) 
                          for value in values]
    else:
        result_set = api_resp[dictionary_key]
        headers = result_set['headers']
        if isinstance(headers[0], dict):
            add_on = headers[0]['columnNames']
            keep_header = headers[1]['columnNames']
            col_ind = 0
            col_count = -1
            add_count = 0
            for col_name in keep_header:
                col_count += 1
                if col_count <= 4:
                    continue
                else:
                    keep_header[col_count] += '_' + add_on[col_ind].replace(' ', '_')
                    add_count += 1
                    if add_count == 3:
                        add_count = 0
                        col_ind += 1
            headers = keep_header
                    
        values = result_set['rowSet']
        name = result_set['name']
        data[name] = [dict(zip(headers, value)) 
                      for value in values]

    return data

    